@@ -1,3 +1,5 @@
+from typing import Any
+
 import yaml
 
 from cekit.descriptor import Descriptor
@@ -24,33 +26,35 @@ class Env(Descriptor):
         super(Env, self).__init__(descriptor)
 
     @property
-    def name(self):
+    def name(self) -> str:
         return self.get("name")
 
     @name.setter
-    def name(self, value):
+    def name(self, value: str):
         self._descriptor["name"] = value
 
     @property
-    def value(self):
+    def value(self) -> Any:
+        # TODO: These should be string, need to update the schema.
         return self.get("value")
 
     @value.setter
-    def value(self, value):
+    def value(self, value: Any):
         self._descriptor["value"] = value
 
     @property
-    def example(self):
+    def example(self) -> Any:
+        # TODO: These should be string, need to update the schema.
         return self.get("example")
 
     @example.setter
-    def example(self, value):
+    def example(self, value: Any):
         self._descriptor["example"] = value
 
     @property
-    def description(self):
+    def description(self) -> str:
         return self.get("description")
 
     @description.setter
-    def description(self, value):
+    def description(self, value: str):
         self._descriptor["description"] = value
