@@ -1,5 +1,7 @@
 import copy
+from typing import Optional
 
+from cekit.cekit_types import RawDescriptor
 from cekit.descriptor import Image
 from cekit.descriptor.image import get_image_schema
 
@@ -9,13 +11,13 @@ overrides_schema["map"]["version"] = {"type": "text"}
 
 
 class Overrides(Image):
-    def __init__(self, descriptor, artifact_dir):
-        self.original_descriptor = copy.deepcopy(descriptor)
-        self._artifact_dir = artifact_dir
+    def __init__(self, descriptor: RawDescriptor, artifact_dir: Optional[str]):
+        self.original_descriptor: RawDescriptor = copy.deepcopy(descriptor)
+        self._artifact_dir: str = artifact_dir
         self.path = artifact_dir
         schema = overrides_schema.copy()
         self.schema = schema
         # calling Descriptor constructor only here (we don't want Image() to mess with schema)
         super(Image, self).__init__(descriptor)
-
+        # TODO: This doesn't set `skip_merging` (though overrides probably aren't merged anywhere)
         self._prepare()
