@@ -5,23 +5,35 @@ import sys
 import tempfile
 from collections import OrderedDict
 from contextlib import closing
+from typing import TYPE_CHECKING, Callable, Dict, List
 from urllib.parse import urlparse
 
 import yaml
 
 from cekit import crypto, version
+from cekit.cekit_types import PathType
 from cekit.config import Config
-from cekit.descriptor.resource import _PlainResource, _PncResource, _UrlResource
+from cekit.descriptor.resource import (
+    Resource,
+    _PlainResource,
+    _PncResource,
+    _UrlResource,
+)
 from cekit.errors import CekitError
 from cekit.generator.base import Generator
 from cekit.tools import copy_recursively, get_brew_url
 
+if TYPE_CHECKING:
+    from cekit.descriptor import Repository
+
 logger = logging.getLogger("cekit")
 config = Config()
 
 
 class OSBSGenerator(Generator):
-    def __init__(self, descriptor_path, target, overrides):
+    def __init__(
+        self, descriptor_path: PathType, target: PathType, overrides: List[str]
+    ):
         self._wipe = True
         super(OSBSGenerator, self).__init__(descriptor_path, target, overrides)
 
@@ -29,6 +41,7 @@ class OSBSGenerator(Generator):
         super(OSBSGenerator, self).init()
 
         self._prepare_osbs_config_file(yaml.safe_dump, "container.yaml")
+        # TODO: Why do we use file.write here and yaml.dump above?
         self._prepare_osbs_config_file(
             lambda contents, file, **kwargs: file.write(contents), "gating.yaml"
         )
@@ -46,7 +59,7 @@ class OSBSGenerator(Generator):
         )
         super(OSBSGenerator, self).generate()
 
-    def _prepare_content_sets(self, content_sets):
+    def _prepare_content_sets(self, content_sets: "Repository"):
         content_sets_f = os.path.join(self.target, "image", "content_sets.yml")
 
         if not os.path.exists(os.path.dirname(content_sets_f)):
@@ -55,11 +68,14 @@ class OSBSGenerator(Generator):
         with open(content_sets_f, "w") as _file:
             yaml.safe_dump(content_sets, _file, default_flow_style=False)
 
-    def _prepare_osbs_config_file(self, writer, config_file):
+    def _prepare_osbs_config_file(
+        self, writer: Callable[..., None], config_file: PathType
+    ) -> None:
         config_path = os.path.join(self.target, "image", config_file)
         config_name = config_file.split(".")[0]
         all_configs = []
 
+        # TODO: Replace `.get` with actual type-safe getters.
         if self.image.get("osbs", {}).get("configuration", {}).get(config_name):
             all_configs.append(
                 self.image.get("osbs", {}).get("configuration", {}).get(config_name)
@@ -92,16 +108,16 @@ class OSBSGenerator(Generator):
         with open(config_path, "w") as _file:
             writer(all_configs[0], _file, default_flow_style=False)
 
-    def prepare_artifacts(self):
+    def prepare_artifacts(self) -> None:
         """Goes through artifacts section of image descriptor
         and fetches all of them
         """
 
         logger.info("Handling artifacts for OSBS...")
         target_dir = os.path.join(self.target, "image")
-        fetch_artifacts_url = []
+        fetch_artifacts_url: List[Dict[str, str]] = []
         fetch_artifacts_pnc = OrderedDict()
-        file_comments = {}
+        file_comments: Dict[str, str] = {}
 
         fetch_domains = config.get("common", "fetch_artifact_domains")
 
@@ -170,6 +186,7 @@ class OSBSGenerator(Generator):
                         fetch_artifacts_url[len(fetch_artifacts_url) - 1].update(
                             {c: artifact[c]}
                         )
+                    patch_source_url(artifact, fetch_artifacts_url)
                     if "description" in artifact:
                         file_comments[artifact["url"]] = artifact["description"]
                     logger.debug(
@@ -198,6 +215,8 @@ class OSBSGenerator(Generator):
                                 "target": os.path.join(artifact["target"]),
                             }
                         )
+                        patch_source_url(artifact, fetch_artifacts_url)
+
                         logger.debug(
                             "Artifact '{}' (as plain) added to fetch-artifacts-url.yaml".format(
                                 artifact["target"]
@@ -273,9 +292,9 @@ class OSBSGenerator(Generator):
 
 
 # Used to modify either the fetch-artifact or fetch-pnc files to add extra human readable information.
-def patch_file(file_comments, file):
+def patch_file(file_comments: Dict[str, str], file: str) -> None:
     # Can't use it as a context manager with plain with as that is >= 3.2
-    with closing(fileinput.input(file, inplace=1)) as input_list:
+    with closing(fileinput.input(file, inplace=True)) as input_list:
         for line in input_list:
             r = [
                 line.replace("\n", " # " + value + "\n")
@@ -286,3 +305,24 @@ def patch_file(file_comments, file):
                 sys.stdout.write(r[0])
             else:
                 sys.stdout.write(line)
+
+
+def patch_source_url(
+    artifact: Resource, fetch_artifacts_url: List[Dict[str, str]]
+) -> None:
+    if "source-url" in artifact:
+        intersected_source_hash = [
+            x for x in crypto.SUPPORTED_SOURCE_HASH_ALGORITHMS if x in artifact
+        ]
+        if not intersected_source_hash:
+            raise CekitError(
+                f"Unable to add source-url for artifact {artifact} as no checksum defined"
+            )
+        logger.debug(
+            f"Found source-url {artifact['source-url']} and checksum markers of {intersected_source_hash}"
+        )
+        fetch_artifacts_url[len(fetch_artifacts_url) - 1].update(
+            {"source-url": artifact["source-url"]}
+        )
+        for c in intersected_source_hash:
+            fetch_artifacts_url[len(fetch_artifacts_url) - 1].update({c: artifact[c]})
